package poc.jdepend.visitor;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;

import jdepend.framework.JDepend;
import jdepend.framework.JavaClass;
import jdepend.framework.JavaPackage;

import org.apache.bcel.Repository;
import org.apache.bcel.classfile.ConstantClass;
import org.apache.bcel.classfile.ConstantPool;
import org.apache.bcel.classfile.DescendingVisitor;
import org.apache.bcel.classfile.EmptyVisitor;


public class JDependWalker {

	private final JDepend jDepend = new JDepend();
	
	public JDependWalker() throws IOException {
		jDepend.addDirectory("build/classes/main");
		jDepend.analyzeInnerClasses(false);
	}
	
	public void visit(final ClassVisitor visitor) {
		final Collection<JavaPackage> result = jDepend.analyze();
		for (final JavaPackage javaPackage : result) {
			final Collection<JavaClass> classes = javaPackage.getClasses();
			for (final JavaClass clazz : classes) {
				visitor.visitClass(clazz.getName());
				for (final String parentClassName : parentsForClass(clazz.getName())) {
					visitor.visitParentClass(clazz.getName(), parentClassName);
				}
				for (final String importedClass : importsForClass(clazz.getName())) {
					visitor.visitImport(clazz.getName(), importedClass);
				}
			}
		}
	}
	
	private Collection<String> importsForClass(final String className) {
		final Collection<String> imports = new ArrayList<String>();
		class DependencyEmitter extends EmptyVisitor {
			private final org.apache.bcel.classfile.JavaClass javaClass;
			public DependencyEmitter(final org.apache.bcel.classfile.JavaClass javaClass) {
				this.javaClass = javaClass;
			}
			@Override
			public void visitConstantClass(final ConstantClass obj) {
				final ConstantPool cp = javaClass.getConstantPool();
				final String bytes = obj.getBytes(cp);
				imports.add(bytes.replaceAll("/", "."));
			}
		}
		org.apache.bcel.classfile.JavaClass javaClass = Repository.lookupClass(className);
		if (javaClass != null) {
			final DependencyEmitter visitor = new DependencyEmitter(javaClass);
			final DescendingVisitor classWalker = new DescendingVisitor(javaClass, visitor);
			classWalker.visit();
		}
		return Collections.unmodifiableCollection(imports);
	}
	
	private Collection<String> parentsForClass(final String className) {
		final Collection<String> parents = new ArrayList<String>();
		org.apache.bcel.classfile.JavaClass javaClass = Repository.lookupClass(className);
		if (javaClass != null) {
			for (org.apache.bcel.classfile.JavaClass interfaceClass : javaClass.getInterfaces()) {
				parents.add(interfaceClass.getClassName());
			}
			parents.add(javaClass.getSuperClass().getClassName());
		}
		return Collections.unmodifiableCollection(parents);
	}
}
